/*
 * Licensed to the University Corporation for Advanced Internet Development, 
 * Inc. (UCAID) under one or more contributor license agreements.  See the 
 * NOTICE file distributed with this work for additional information regarding
 * copyright ownership. The UCAID licenses this file to You under the Apache 
 * License, Version 2.0 (the "License"); you may not use this file except in 
 * compliance with the License.  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package edu.internet2.middleware.shibboleth.idp.ext.securityconfig;

import java.util.List;

import org.opensaml.xml.Configuration;
import org.opensaml.xml.security.BasicSecurityConfiguration;
import org.opensaml.xml.security.SecurityConfiguration;
import org.opensaml.xml.security.keyinfo.KeyInfoCredentialResolver;
import org.opensaml.xml.security.keyinfo.KeyInfoGeneratorFactory;
import org.opensaml.xml.security.keyinfo.KeyInfoGeneratorManager;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.InitializingBean;

import edu.internet2.middleware.shibboleth.common.config.OpensamlConfigBean;

/**
 * A simple Spring initializing bean that may be used to do some custom post-initialization configuration of security
 * settings for the OpenSAML library. Must be called after OpenSAML is initialized, for example via
 * {@link OpensamlConfigBean}.
 */
public class OpensamlCustomSecurityConfigBean implements InitializingBean {

    /** Class logger. */
    private final Logger log = LoggerFactory.getLogger(OpensamlCustomSecurityConfigBean.class);

    // Signature-related properties.
    
    /** Signing algorithm to use with an RSA private key. */
    private String signatureAlgorithmRSA;

    /** Signing algorithm to use with a DSA private key. */
    private String signatureAlgorithmDSA;

    /** Signing algorithm to use with an EC private key. */
    private String signatureAlgorithmEC;

    /** Signing algorithm to use with an AES secret key. */
    private String signatureAlgorithmAES;

    /** Signing algorithm to use with an DESede secret key. */
    private String signatureAlgorithmDESede;

    /** Signature digest method. */
    private String signatureReferenceDigestMethod;

    /** Signature canonicalization method. */
    private String signatureCanonicalizationAlgorithm;

    /** Signature HMAC output length. */
    private Integer signatureHMACOutputLength;

    // Encryption-related properties.
    
    /** Encryption bulk data encryption algorithm. */
    private String autoGeneratedDataEncryptionKeyAlgorithm;

    /** Encryption RSA key transport algorithm for AES keys. */
    private String keyTransportEncryptionAlgorithmRSAofAES;

    /** Encryption RSA key transport algorithm for DESede keys. */
    private String keyTransportEncryptionAlgorithmRSAofDESede;
    
    // KeyInfo-related properties

    /** The default KeyInfo credential resolver instance. */
    private KeyInfoCredentialResolver defaultKeyInfoCredentialResolver;

    /** The KeyInfo generator factory instances. */
    private List<KeyInfoGeneratorFactory> keyInfoGeneratorFactories;

    /** {@inheritDoc} */
    public void afterPropertiesSet() throws Exception {
        SecurityConfiguration config = Configuration.getGlobalSecurityConfiguration();

        if (config instanceof BasicSecurityConfiguration) {
            BasicSecurityConfiguration basicConfig =
                    (BasicSecurityConfiguration) Configuration.getGlobalSecurityConfiguration();

            handleSignatureParams(basicConfig);
            handleEncryptionParams(basicConfig);
            handleKeyInfoCredentialResolverParams(basicConfig);
            handleKeyInfoGeneratorParams(basicConfig);

        } else {
            log.warn("Configuration.getGlobalSecurityConfiguration did not produce an instance of " +
                    "BasicSecurityConfiguration, could not apply custom security configuration settings: {}", 
                            config == null ? "null" : config.getClass().getName());
        }

    }

    /**
     * Get the XML Signature signing algorithm to use with an RSA private key.
     * 
     * @return  RSA private key signing algorithm
     */
    public String getSignatureAlgorithmRSA() {
        return signatureAlgorithmRSA;
    }

    /**
     * Set the XML Signature signing algorithm to use with an RSA private key.
     * 
     * @param algorithm  RSA private key signing algorithm
     */
    public void setSignatureAlgorithmRSA(String algorithm) {
        signatureAlgorithmRSA = algorithm;
    }

    /**
     * Get the XML Signature signing algorithm to use with a DSA private key.
     * 
     * @return  DSA private key signing algorithm
     */
    public String getSignatureAlgorithmDSA() {
        return signatureAlgorithmDSA;
    }

    /**
     * Set the XML Signature signing algorithm to use with a DSA private key.
     * 
     * @param algorithm  DSA private key signing algorithm
     */
    public void setSignatureAlgorithmDSA(String algorithm) {
        signatureAlgorithmDSA = algorithm;
    }

    /**
     * Get the XML Signature signing algorithm to use with an EC private key.
     * 
     * @return  EC private key signing algorithm
     */
    public String getSignatureAlgorithmEC() {
        return signatureAlgorithmEC;
    }

    /**
     * Set the XML Signature signing algorithm to use with an EC private key.
     * 
     * @param algorithm  EC private key signing algorithm
     */
    public void setSignatureAlgorithmEC(String algorithm) {
        signatureAlgorithmEC = algorithm;
    }

    /**
     * Get the XML Signature signing algorithm to use with an AES secret key.
     * 
     * @return  AES secret key signing algorithm
     */
    public String getSignatureAlgorithmAES() {
        return signatureAlgorithmAES;
    }

    /**
     * Set the XML Signature signing algorithm to use with an AES secret key.
     * 
     * @param algorithm  AES secret key signing algorithm
     */
    public void setSignatureAlgorithmAES(String algorithm) {
        signatureAlgorithmAES = algorithm;
    }

    /**
     * Get the XML Signature signing algorithm to use with a 3DES secret key.
     * 
     * @return  3DES secret key signing algorithm
     */
    public String getSignatureAlgorithmDESede() {
        return signatureAlgorithmDESede;
    }

    /**
     * Set the XML Signature signing algorithm to use with a 3DES secret key.
     * 
     * @param algorithm  3DES secret key signing algorithm
     */
    public void setSignatureAlgorithmDESede(String algorithm) {
        signatureAlgorithmDESede = algorithm;
    }

    /**
     * Get the XML Signature digest method.
     * 
     * @return  digest method
     */
    public String getSignatureReferenceDigestMethod() {
        return signatureReferenceDigestMethod;
    }

    /**
     * Set the XML Signature digest method.
     * 
     * @param algorithm digest method
     */
    public void setSignatureReferenceDigestMethod(String algorithm) {
        signatureReferenceDigestMethod = algorithm;
    }

    /**
     * Get the XML Signature canonicalization method.
     * 
     * @return  canonicalization method
     */
    public String getSignatureCanonicalizationAlgorithm() {
        return signatureCanonicalizationAlgorithm;
    }

    /**
     * Set the XML Signature canonicalization method.
     * 
     * @param algorithm  canonicalization method
     */
    public void setSignatureCanonicalizationAlgorithm(String algorithm) {
        signatureCanonicalizationAlgorithm = algorithm;
    }

    /**
     * Get the XML Signature HMAC OutputLength.
     * 
     * @return  HMAC OutputLength
     */
    public Integer getSignatureHMACOutputLength() {
        return signatureHMACOutputLength;
    }

    /**
     * Set the XML Signature HMAC OutputLength.
     * 
     * @param length  HMAC OutputLength
     */
    public void setSignatureHMACOutputLength(Integer length) {
        signatureHMACOutputLength = length;
    }

    /**
     * Get the XML Encryption bulk data encryption algorithm.
     * 
     * @return  bulk data encryption algorithm
     */
    public String getAutoGeneratedDataEncryptionKeyAlgorithm() {
        return autoGeneratedDataEncryptionKeyAlgorithm;
    }

    /**
     * Set the XML Encryption bulk data encryption algorithm.
     * 
     * @param algorithm  bulk data encryption algorithm
     */
    public void setAutoGeneratedDataEncryptionKeyAlgorithm(String algorithm) {
        autoGeneratedDataEncryptionKeyAlgorithm = algorithm;
    }

    /**
     * Get the XML Encryption RSA key transport algorithm for AES keys.
     * 
     * @return  RSA key transport algorithm for AES keys
     */
    public String getKeyTransportEncryptionAlgorithmRSAofAES() {
        return keyTransportEncryptionAlgorithmRSAofAES;
    }

    /**
     * Set the XML Encryption RSA key transport algorithm for AES keys.
     * 
     * @param algorithm  RSA key transport algorithm for AES keys
     */
    public void setKeyTransportEncryptionAlgorithmRSAofAES(String algorithm) {
        keyTransportEncryptionAlgorithmRSAofAES = algorithm;
    }

    /**
     * Get the XML Encryption RSA key transport algorithm for 3DES keys.
     * 
     * @return  RSA key transport algorithm for 3DES keys
     */
    public String getKeyTransportEncryptionAlgorithmRSAofDESede() {
        return keyTransportEncryptionAlgorithmRSAofDESede;
    }

    /**
     * Set the XML Encryption RSA key transport algorithm for 3DES keys.
     * 
     * @param algorithm  RSA key transport algorithm for 3DES keys
     */
    public void setKeyTransportEncryptionAlgorithmRSAofDESede(String algorithm) {
        keyTransportEncryptionAlgorithmRSAofDESede = algorithm;
    }

    /**
     * Get the default {@link KeyInfoCredentialResolver} instance to use.
     * 
     * @return default KeyInfoCredentialResolver instance
     */
    public KeyInfoCredentialResolver getDefaultKeyInfoCredentialResolver() {
        return defaultKeyInfoCredentialResolver;
    }

    /**
     * Set the default {@link KeyInfoCredentialResolver} instance to use.
     * 
     * @param resolver default KeyInfoCredentialResolver instance
     */
    public void setDefaultKeyInfoCredentialResolver(KeyInfoCredentialResolver resolver) {
        defaultKeyInfoCredentialResolver = resolver;
    }

    /**
     * Get the {@link KeyInfoGeneratorFactory} instances to install.
     * 
     * @return KeyInfoGeneratorFactory instances to install
     */
    public List<KeyInfoGeneratorFactory> getKeyInfoGeneratorFactories() {
        return keyInfoGeneratorFactories;
    }

    /**
     * Set the {@link KeyInfoGeneratorFactory} instances to install.
     * 
     * @param factories KeyInfoGeneratorFactory instances to install
     */
    public void setKeyInfoGeneratorFactories(List<KeyInfoGeneratorFactory> factories) {
        keyInfoGeneratorFactories = factories;
    }

    /**
     * Process the parameters related to signatures.
     * 
     * @param basicConfig the security configuration
     */
    protected void handleSignatureParams(BasicSecurityConfiguration basicConfig) {
        if (getSignatureAlgorithmRSA() != null) {
            basicConfig.registerSignatureAlgorithmURI("RSA", getSignatureAlgorithmRSA());
        }

        if (getSignatureAlgorithmDSA() != null) {
            basicConfig.registerSignatureAlgorithmURI("DSA", getSignatureAlgorithmDSA());
        }

        if (getSignatureAlgorithmEC() != null) {
            basicConfig.registerSignatureAlgorithmURI("EC", getSignatureAlgorithmEC());
        }

        if (getSignatureAlgorithmAES() != null) {
            basicConfig.registerSignatureAlgorithmURI("AES", getSignatureAlgorithmAES());
        }

        if (getSignatureAlgorithmDESede() != null) {
            basicConfig.registerSignatureAlgorithmURI("DESede", getSignatureAlgorithmDESede());
        }

        if (getSignatureReferenceDigestMethod() != null) {
            basicConfig.setSignatureReferenceDigestMethod(getSignatureReferenceDigestMethod());
        }

        if (getSignatureCanonicalizationAlgorithm() != null) {
            basicConfig.setSignatureCanonicalizationAlgorithm(getSignatureCanonicalizationAlgorithm());
        }

        if (getSignatureHMACOutputLength() != null) {
            basicConfig.setSignatureHMACOutputLength(getSignatureHMACOutputLength());
        }
    }

    /**
     * Process the parameters related to encryption.
     * 
     * @param basicConfig the security configuration
     */
    protected void handleEncryptionParams(BasicSecurityConfiguration basicConfig) {
        if (getKeyTransportEncryptionAlgorithmRSAofAES() != null) {
            basicConfig.registerKeyTransportEncryptionAlgorithmURI("RSA", null, "AES",
                    getKeyTransportEncryptionAlgorithmRSAofAES());
        }

        if (getKeyTransportEncryptionAlgorithmRSAofDESede() != null) {
            basicConfig.registerKeyTransportEncryptionAlgorithmURI("RSA", null, "DESede",
                    getKeyTransportEncryptionAlgorithmRSAofDESede());
        }

        if (getAutoGeneratedDataEncryptionKeyAlgorithm() != null) {
            basicConfig.setAutoGeneratedDataEncryptionKeyAlgorithmURI(getAutoGeneratedDataEncryptionKeyAlgorithm());
        }
    }

    /**
     * Process the parameters related to KeyInfo credential resolution.
     * 
     * @param basicConfig the security configuration
     */
    protected void handleKeyInfoCredentialResolverParams(BasicSecurityConfiguration basicConfig) {
        if (getDefaultKeyInfoCredentialResolver() != null) {
            basicConfig.setDefaultKeyInfoCredentialResolver(getDefaultKeyInfoCredentialResolver());
        }
    }

    /**
     * Process the parameters related to KeyInfo generation.
     * 
     * @param basicConfig the security configuration
     */
    protected void handleKeyInfoGeneratorParams(BasicSecurityConfiguration basicConfig) {
        if (getKeyInfoGeneratorFactories() != null) {
            KeyInfoGeneratorManager manager = basicConfig.getKeyInfoGeneratorManager().getDefaultManager();
            for (KeyInfoGeneratorFactory factory : getKeyInfoGeneratorFactories()) {
                if (factory != null) {
                    manager.registerFactory(factory);
                }
            }
        }
    }

}